


/* Inclusion du fichier  pic.h  , qui dfinit, entre autres, les registres du microcontrleur PIC16F84. */
#include <pic.h>



#define DIRECTION_TRANSMISSIONS_DONNEES TRISA0



#define BROCHE_TRANSMISSIONS_DONNEES RA0



/* Configuration du microcontrleur PIC16F84A : Pas de protection du code, et utilisation d'un rseau "R-C", comme oscillateur. */
__CONFIG(0x3FFB);



void Delai(unsigned short Temps);
void Generer(unsigned char e);
unsigned char Scruter(void);
void Transmettre(unsigned char c);
void main(void);



/* Fonction qui sert  faire perdre son temps au microcontrleur PIC16F84. */
void Delai(unsigned short Temps)
{
	int k;	

	for(k = 0; k < Temps; k++);
}



/* Fonction gnrant un signal  0  , ou un signal  1 . */
void Generer(unsigned char e)
{
	unsigned char k;

	if(e == 0)
	{
		BROCHE_TRANSMISSIONS_DONNEES = 1;
		Delai(8);
	}
	else
	{
		BROCHE_TRANSMISSIONS_DONNEES = 1;
		Delai(4);	
	}
	if(e == 0)
	{
		BROCHE_TRANSMISSIONS_DONNEES = 0;
		Delai(4);
	}
	else
	{
		BROCHE_TRANSMISSIONS_DONNEES = 0;
		Delai(8);
	}
}



/* Fonction scrutant l'ensemble des 16 boutons-poussoirs de la tlcommande, ligne aprs ligne. */
unsigned char Scruter(void)
{
	unsigned char Ligne;
	unsigned char Colonne;

	Colonne = 7;

	while(1)
	{
		for(Ligne = 0; Ligne < 4; Ligne++)
		{
			PORTB = ~(1 << Ligne);

			if(!RB4)
			{
				Colonne = 0;
			}
			if(!RB5)
			{
				Colonne = 1;
			}
			if(!RB6)
			{
				Colonne = 2;
			}
			if(!RB7)
			{
				Colonne = 3;
			}
			if(Colonne =! 7)
			{
				return((4 * Ligne) + Colonne);
			}
		}
	}
	return 0;						
}



/* Fonction transmettant le code correspondant au bouton-poussoir de la tlcommande appuy. */
void Transmettre(unsigned char c)
{
	unsigned char k;

	unsigned char l;

	unsigned char b[4] = {0};
	
	for(k = 0; k < 4; k++)
	{
		if((c % 2) == 1)
		{
			b[k] = 1;
		}
		c = (c / 2);
	}

	for(l = 0; l < 5; l++)
	{
		Generer(1);
		Generer(1);
		Generer(0);
		Generer(1);
		Generer(0);
		Generer(0);
		Generer(1);
		Generer(0);
		Generer(b[3]);
		Generer(b[2]);
		Generer(b[1]);
		Generer(b[0]);
		Generer(1);
		Generer(1);
		Generer(0);
		Generer(1);

		Delai(100);		
	}
}



/* Fonction principale. */
void main(void)
{
	unsigned char k;

	TRISA = 0xFE;

	TRISB = 0xF0;



	DIRECTION_TRANSMISSIONS_DONNEES = 0;	/* Configuration de la broche de transmission des donnes en tant que sortie de type  Tout ou rien . */



	OPTION_REG = 0b01000000;	/* OPTION REGISTER : ~RBPU = 0, Activation des rsistances de pull-up du PORT B,
													 INTEDG = 0, Non utilis,
													 T0CS = 0, Non utilis,
													 T0SE = 0, Non utilis,
													 PSA = 0, Non utilis,
													 PS2 = 0,
													 PS1 = 0,
													 PS0 = 0, Non utilis. */

	INTCON = 0b00000000;	/* Dsactivation gnrale des interruptions. */


	while(1)
	{
		BROCHE_TRANSMISSIONS_DONNEES = 0;

		k = Scruter();

		Transmettre(k);	/* Appel de la fonction de scrutation de l'ensemble des 16 boutons-poussoirs de la tlcommande. */
	}
}


 