


/* Inclusion du fichier  pic.h  , qui dfinit, entre autres, les registres du microcontrleur PIC16F84. */
#include <pic.h>



#define DIRECTION_RECEPTIONS_DONNEES TRISB0



#define BROCHE_RECEPTIONS_DONNEES RB0



/* Configuration du microcontrleur PIC16F84A : Pas de protection du code, et utilisation d'un rseau  R - C  , comme oscillateur. */
__CONFIG(0x3FFB);



void Delai(unsigned short Temps);
void main(void);




/* Fonction qui sert  faire perdre son temps au microcontrleur PIC16F84. */
void Delai(unsigned short Temps)
{
	unsigned short k;	

	for(k = 0; k < Temps; k++);
}



/* Fonction principale. */
void main(void) 
{ 
	TRISA = 0xF2;
	PORTA = 0x00;

	TRISB = 0x01;
	PORTB = 0x00;



	DIRECTION_RECEPTIONS_DONNEES = 1;	/* Configuration de la broche de rception des donnes ent tant qu'entre de type  Tout ou rien . */



	OPTION_REG = 0b11000000;	/* OPTION REGISTER : ~RBPU = 1, Dsactivation des rsistances de pull-up du PORT B,
													 INTEDG = 1, Interruption provoque par les front montant transmis sur la broche  RB0 - INT ,
													 T0CS = 0, Horloge du TIMER 0 fournie par le microcontrleur PIC16F84,
													 T0SE = 0, Non utilis,
													 PSA = 0, Prdiviseur en lien avec le TIMER 0,
													 PS2 = 0,
													 PS1 = 0,
													 PS0 = 0, Prdivision par 2. */

	INTCON = 0b10110000;	/* Activation de l'interruption qui provient du dbordement du TIMER 0, et de l'interruption qui provient d'un front montant transmis sur la broche  RB0 - INT . */
	TMR0 = 0x00;	/* Remise  zro du TIMER 0. */

	GIE = 1;	/* Activation gnrale des interruptions du microcontrleur PIC16F84. */

	/* Boucle sans fin. */
	while(1)
	{
		
	}

}


	
/* Routines d'interruptions. */
static void	interrupt isr(void)
{
	static unsigned char Compteur = 0;
	
	static unsigned char Donnee1 = 0;
	
	static unsigned char Donnee2 = 0;
	
	static unsigned char Reception = 0;

	if(T0IF)	/* Si l'interruption provient du dbordement du TIMER 0 */
	{
		if(Reception == 1) /* Si un signal radio qui contenait moins de seize impulsions suffisamment proches avait t reu */
		{
			Reception = 0;
			Donnee1 = 0;
			Donnee2 = 0;
			Compteur = 0;
		}
	}
	if(INTF)	/* Si l'interruption provient d'un front montant transmis sur la broche  RB0 - INT  , c'est--dire si un signal radio est en train d'tre reu */
	{
		Delai(3);	/* Attente du passage ventuel  zro avant de lire l'tat de la broche  RB0 - INT  , aprs l'arrive du front montant qui a provoy l'interruption. */

		TMR0 = 0x00;	/* Remise  zro du TIMER 0. */

		Reception = 1;

		T0IE = 1;

		if(Compteur < 16)
		{
			Compteur++;
			if(Compteur < 9)
			{
				Donnee1 <<= 1; /* On dcale l'ensemble des bits d'un rang vers la gauche. */
	
				if(BROCHE_RECEPTIONS_DONNEES == 0)	/* Si la broche  RB0 - INT  est repasse   0  , l'impulsion tait une impulsion courte, qui codait un  1  */
				{
					Donnee1 |= 1; /* On insre le bit de poids faible   1 . */
				}		
			}
			else
			{
				Donnee2 <<= 1; /* On dcale l'ensemble des bits d'un rang vers la gauche. */
	
				if(BROCHE_RECEPTIONS_DONNEES == 0)	/* Si la broche  RB0 - INT  est repasse   0  , l'impulsion tait une impulsion courte, qui codait un  1  */
				{
					Donnee2 |= 1; /* On insre le bit de poids faible   1 . */
				}
			}
			if((Donnee1 == 0b11010010) && ((Donnee2 & 0b00001111) == 0b00001101) && (Compteur == 16)) /* Le signal radio est conforme, et il tait compos d'exactement seize impulsions. */
			{
				Donnee2 >>= 4;
				Donnee2 &= 0x0F;
				/* Effectuer une action ici, ventuellement en fonction du code de la touche de la tlcommande radio appuye, contenu dans la variable statique  Donnee2 . */
				Compteur = 0;
				Donnee1 = 0;
				Donnee2 = 0;
				Reception = 0;
			}
		}
		else
		{
			Compteur = 0;
			Donnee1 = 0;
			Donnee2 = 0;
			Reception = 0;
		}

		INTF = 0;	/* Rarmement de l'interruption qui provient d'un front montant transmis sur la broche  RB0 - INT . */
	}
	INTF = 0;	/* Rarmement de l'interruption qui provient d'un front montant transmis sur la broche  RB0 - INT . */	

	T0IF = 0;	/* Rarmement de l'interruption qui provient du dbordement du TIMER 0. */
}


